@@ -5,7 +5,7 @@ use warnings;
 use strict;
 use 5.014;
 
-use List::MoreUtils qw( any none );
+use List::MoreUtils qw( any );
 
 use Term::Choose       qw();
 use Term::Choose::Util qw();
@@ -16,18 +16,18 @@ use App::DBBrowser::GetContent::Filter;
 
 
 sub new {
-    my ( $class, $info, $options, $data ) = @_;
+    my ( $class, $info, $options, $d ) = @_;
     my $sf = {
         i => $info,
         o => $options,
-        d => $data,
+        d => $d
     };
     bless $sf, $class;
 }
 
 
 sub search_and_replace {
-    my ( $sf, $sql, $filter_str ) = @_;
+    my ( $sf, $sql, $bu_insert_into_args, $filter_str, $back  ) = @_;
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
     my $tf = Term::Form->new( $sf->{i}{tf_default} );
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
@@ -38,9 +38,10 @@ sub search_and_replace {
     my $saved = $ax->read_json( $sf->{i}{f_search_and_replace} ) // {};
     my $all_sr_groups = [];
     my $used_names = [];
-
+    $sf->{s_back} = $back;
     my @bu;
     my ( $hidden, $add ) = ( 'Your choice:', '  ADD search & replace' );
+    my $available = [ sort { $a cmp $b } keys %$saved  ];
 
     ADD_SEARCH_AND_REPLACE: while ( 1 ) {
         my @tmp_info = ( '', $filter_str );
@@ -49,43 +50,37 @@ sub search_and_replace {
                 push @tmp_info, '  s/' . join( '/', @$sr_single ) . ';';
             }
         }
-        #for my $i ( 0 .. $#$all_sr_groups ) {
-        #    push @tmp_info, '  ' . $used_names->[$i];
-        #    my $sr_group = $all_sr_groups->[$i];
-        #    for my $sr_single ( @$sr_group ) {
-        #        push @tmp_info, '    s/' . join( '/', @$sr_single ) . ';';
-        #    }
-        #}
         push @tmp_info, '';
         my @pre = ( $hidden, undef, $sf->{i}{_confirm}, $add );
-        my $available = [];
-        for my $name ( sort { $a cmp $b } keys %$saved ) {
-            if ( none { $name eq $_ } @$used_names ) {
-                push @$available, $name;
+        my $prefixed_available = [];
+        for my $name ( @$available ) {
+            if ( any { $_ eq $name } @$used_names ) {
+                push @$prefixed_available, '- ' . $name . ' (used)';
+            }
+            else {
+                push @$prefixed_available, '- ' . $name;
             }
         }
-        my $prefixed_available = [ map { '- ' . $_ } @$available ];
         my $menu = [ @pre, @$prefixed_available ];
-        my $count_static_rows = @tmp_info;
-        my $info = $cf->__get_filter_info( $sql, $count_static_rows, [ $hidden, $sf->{i}{_back}, $sf->{i}{_confirm}, $add ], $prefixed_available, 1 ) . join( "\n", @tmp_info );
+        my $info = $cf->__get_filter_info( $sql, join( "\n", @tmp_info ) );
         # Choose
         my $idx = $tc->choose(
             $menu,
-            { %{$sf->{i}{lyt_v}}, info => $info, prompt => '', default => 1, index => 1, undef => $sf->{i}{_back},
-              keep => $sf->{i}{fi}{keep} }
+            { %{$sf->{i}{lyt_v}}, info => $info, prompt => '', default => 1, index => 1, undef => $sf->{i}{_back} }
         );
         if ( ! defined $idx || ! defined $menu->[$idx] ) {
             if ( @bu ) {
                 ( $used_names, $all_sr_groups ) = @{pop @bu};
                 next ADD_SEARCH_AND_REPLACE;
             }
-            $sql->{insert_into_args} = [ map { [ @$_ ] } @{$sf->{i}{fi}{bu_insert_into_args}} ];
+            $sql->{insert_into_args} = [ map { [ @$_ ] } @{$bu_insert_into_args} ];
             return;
         }
         my $choice = $menu->[$idx];
         if ( $choice eq $hidden ) {
             $sf->__history( $sql );
             $saved = $ax->read_json( $sf->{i}{f_search_and_replace} ) // {};
+            $available = [ sort { $a cmp $b } keys %$saved  ];
             next ADD_SEARCH_AND_REPLACE;
         }
         elsif ( $choice eq $sf->{i}{_confirm} ) {
@@ -96,7 +91,7 @@ sub search_and_replace {
             APPLY_TO_COS: while ( 1 ) {
                 my $ok = $sf->__apply_to_cols( $sql, \@tmp_info, $header, $all_sr_groups );
                 if ( ! $ok ) {
-                    $sql->{insert_into_args} = [ map { [ @$_ ] } @{$sf->{i}{fi}{bu_insert_into_args}} ];
+                    $sql->{insert_into_args} = [ map { [ @$_ ] } @{$bu_insert_into_args} ];
                     next ADD_SEARCH_AND_REPLACE;
                 }
                 my $header_changed = 0;
@@ -111,18 +106,16 @@ sub search_and_replace {
                     my $menu = [ undef, $yes, $no ];
                     my @tmp_info_addition = ( 'Header: ' . join( ', ', @{$sql->{insert_into_args}[0]} ), ' ' );
                     push @tmp_info, @tmp_info_addition;
-                    my $count_static_rows = @tmp_info;
-                    my $info = $cf->__get_filter_info( $sql, $count_static_rows, [ $sf->{i}{fi}{back} ], [ $yes, $no ], 1 ) . join( "\n", @tmp_info );
+                    my $info = $cf->__get_filter_info( $sql, join( "\n", @tmp_info ) );
                     # Choose
                     my $idx = $tc->choose(
                         $menu,
-                        { %{$sf->{i}{lyt_v}}, info => $info, prompt => 'Restore header?', default => 0, index => 1, undef => $sf->{i}{fi}{back},
-                        keep => $sf->{i}{fi}{keep} }
+                        { %{$sf->{i}{lyt_v}}, info => $info, prompt => 'Restore header?', default => 0, index => 1, undef => $sf->{s_back} }
                     );
                     if ( ! defined $idx || ! defined $menu->[$idx] ) {
                         my $pop_count = @tmp_info_addition;
                         splice @tmp_info, -$pop_count;
-                        $sql->{insert_into_args} = [ map { [ @$_ ] } @{$sf->{i}{fi}{bu_insert_into_args}} ];
+                        $sql->{insert_into_args} = [ map { [ @$_ ] } @{$bu_insert_into_args} ];
                         next APPLY_TO_COS;
                     }
                     my $choice = $menu->[$idx];
@@ -145,15 +138,14 @@ sub search_and_replace {
                     [ $nr . ' Replacement', ],
                     [ $nr . ' Modifiers',   ];
             }
-            my $count_static_rows = @tmp_info + 1; # tmp_info, prompt
             my $back = $sf->{i}{back} . '   ';
 
             SUBSTITUTION: while ( 1 ) {
-                my $info = $cf->__get_filter_info( $sql, $count_static_rows, [ $back, $sf->{i}{confirm} ], $fields, 1 ) . join( "\n", @tmp_info );
+                my $info = $cf->__get_filter_info( $sql, join( "\n", @tmp_info ) );
                 # Fill_form
                 my $form = $tf->fill_form(
                     $fields,
-                    { info => $info, prompt => $prompt, auto_up => 2, confirm => $sf->{i}{confirm}, keep => $sf->{i}{fi}{keep},
+                    { info => $info, prompt => $prompt, auto_up => 2, confirm => $sf->{i}{confirm},
                       back => $back, skip_items => $skip_regex }
                 );
                 if ( ! defined $form ) {
@@ -200,37 +192,17 @@ sub __apply_to_cols {
     my $tu = Term::Choose::Util->new( $sf->{i}{tcu_default} );
     my $cf = App::DBBrowser::GetContent::Filter->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $aoa = $sql->{insert_into_args};
-    my $count_static_rows = @$tmp_info + 1; # info_count and cs_label
-    my $key_1 = 'search&replace';
-    my $key_2;
-    for my $sr_group ( @$all_sr_groups ) {
-        for my $sr ( @$sr_group ) {
-            $key_2 .= join( '', @$sr );
-        }
-    }
-    my $prev_chosen = $sf->{i}{fi}{prev_chosen_cols}{$key_1}{$key_2} // [];
-    my $mark;
-    if ( @$prev_chosen && @$prev_chosen < @$header ) {
-        $mark = [];
-        for my $i ( 0 .. $#{$header} ) {
-            if ( any { $_ eq $header->[$i] } @$prev_chosen ) {
-                push @$mark, $i;
-            }
-        }
-        $mark = undef if @$mark != @$prev_chosen;
-    }
-    my $info = $cf->__get_filter_info( $sql, $count_static_rows, [ $sf->{i}{fi}{back}, $sf->{i}{ok} ], $header, undef ) . join( "\n", @$tmp_info );
+    my $info = $cf->__get_filter_info( $sql, join( "\n", @$tmp_info ) );
     # Choose
     my $col_idxs = $tu->choose_a_subset(
         $header,
-        { cs_label => 'Apply to: ', info => $info, layout => 0, all_by_default => 1, index => 1, keep => $sf->{i}{fi}{keep},
-        confirm => $sf->{i}{ok}, back => $sf->{i}{fi}{back}, mark => $mark }
+        { cs_label => 'Apply to: ', info => $info, layout => 0, all_by_default => 1, index => 1,
+        confirm => $sf->{i}{ok}, back => $sf->{s_back} }
     );
     $cf->__print_busy_string();
     if ( ! defined $col_idxs ) {
         return;
     }
-    $sf->{i}{fi}{prev_chosen_cols}{$key_1}{$key_2} = [ @{$header}[@$col_idxs] ];
     $sf->__execute_substitutions( $aoa, $col_idxs, $all_sr_groups );
     $sql->{insert_into_args} = $aoa;
     return 1;
@@ -289,6 +261,8 @@ sub __history {
     my $tf = Term::Form->new( $sf->{i}{tf_default} );
     my $tu = Term::Choose::Util->new( $sf->{i}{tcu_default} );
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my $separator_key = ' ';
+    my $skip_regex = qr/^\Q${separator_key}\E\z/;
     my $old_idx_history = 0;
 
     HISTORY: while ( 1 ) {
@@ -314,10 +288,12 @@ sub __history {
         }
         my $choice = $menu->[$idx];
         if ( $choice eq $add ) {
+            my $separator_key = ' ';
+            my $skip_regex = qr/^\Q${separator_key}\E\z/;
             my $fields = [];
             for my $nr ( 1 .. 9 ) {
                 push @$fields,
-                    [ ' ',                  ],
+                    [ $separator_key,       ],
                     [ $nr . ' Pattern',     ],
                     [ $nr . ' Replacement', ],
                     [ $nr . ' Modifiers',   ];
@@ -328,7 +304,7 @@ sub __history {
                 my $form = $tf->fill_form(
                     $fields,
                     { prompt => 'Add s_&_r:', auto_up => 2, clear_screen => 1, info => $top,
-                      section_separators => [ grep { ! ( $_ % 4 ) } 0 .. $#$fields ],
+                      skip_items => $skip_regex,
                       confirm => '  ' . $sf->{i}{confirm}, back => '  ' . $sf->{i}{back} . '   ' }
                 );
                 if ( ! defined $form ) {
@@ -386,7 +362,7 @@ sub __history {
 
                 EDIT_ENTRY: while ( 1 ) {
                     my $fields = [
-                        [ ' ' ],
+                        [ $separator_key   ],
                         [ '  Pattern',     ],
                         [ '  Replacement', ],
                         [ '  Modifiers',   ]
@@ -396,11 +372,11 @@ sub __history {
                         my ( $pattern, $replacement, $modifiers ) = @$sr_single;
                         $c++;
                         push @$fields,
-                            [ ' ' ],
+                            [ $separator_key ],
                             [ $c . ' Pattern',     $pattern     ],
                             [ $c . ' Replacement', $replacement ],
                             [ $c . ' Modifiers',   $modifiers   ],
-                            [ ' ' ],
+                            [ $separator_key ],
                             [ '  Pattern',     ],
                             [ '  Replacement', ],
                             [ '  Modifiers',   ];
@@ -413,7 +389,7 @@ sub __history {
                     my $form = $tf->fill_form(
                         $fields,
                         { prompt => "Edit \"$name\":", auto_up => 2, clear_screen => 1, info => $info,
-                          section_separators => [ grep { ! ( $_ % 4 ) } 0 .. $#$fields ],
+                          skip_items => $skip_regex,
                           confirm => '  ' . $sf->{i}{confirm}, back => '  ' . $sf->{i}{back} . '   ' }
                     );
                     if ( ! defined $form ) {
@@ -534,6 +510,7 @@ sub __get_entry_name {
             if ( $count > 1 ) {
                 $new_name = undef;
             }
+            $name_default = $new_name;
             $count++;
             next NAME;
         }
